@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -16,9 +16,9 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include "application.h"
+#include "config.h"
 #include "extension_set.h"
 #include "log.h"
-#include <mobius/database/connection_pool.h>
 #include <mobius/exception.inc>
 #include <mobius/io/path.h>
 #include <mobius/io/folder.h>
@@ -29,9 +29,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-namespace mobius
-{
-namespace core
+namespace mobius::core
 {
 namespace
 {
@@ -45,15 +43,15 @@ static const std::string id_ = "mobiusft";
 static const std::string name_ = "Mobius Forensic Toolkit";
 
 //! \brief application version
-static const std::string version_ = "1.33";
+static const std::string version_ = "2.3";
 
 //! \brief application title
-static const std::string title_ = "Mobius v1.33";
+static const std::string title_ = "Mobius v2.3";
 
 //! \brief application copyright
-static const std::string copyright_ = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar";
+static const std::string copyright_ = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Eduardo Aguiar";
 
-//! \brief application OS name  
+//! \brief application OS name
 static const std::string os_name_ = "x86_64-pc-linux-gnu";
 
 //! \brief data folder path
@@ -65,9 +63,6 @@ static std::string config_folder_;
 //! \brief cache folder path
 static std::string cache_folder_;
 
-//! \brief module's connection pool
-static mobius::database::connection_pool connection_pool_;
-
 //! \brief C++ extensions
 static extension_set extensions_;
 
@@ -84,7 +79,7 @@ static std::atomic_bool is_running_ {false};
 //! \return absolute path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static std::string
-join_path (const std::string& abs_path, const std::string& rel_path)
+_join_path (const std::string& abs_path, const std::string& rel_path)
 {
   auto path = mobius::io::path (abs_path);
   return to_string (path.get_child_by_path (rel_path));
@@ -97,7 +92,7 @@ void
 _init ()
 {
   const char *home = getenv ("HOME");
-      
+
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Config folder
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -114,12 +109,13 @@ _init ()
   config_folder.create ();
 
   set_logfile_path (config_folder_ + "/mobius.log");
-      
+  set_configfile_path (config_folder_ + "/mobiusft.sqlite");
+
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Cache folder
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   const char *xdg_cache_home = getenv ("XDG_CACHE_HOME");
-      
+
   if (xdg_cache_home)
     cache_folder_ = xdg_cache_home;
 
@@ -130,7 +126,7 @@ _init ()
 
   auto cache_folder = mobius::io::new_folder_by_path (cache_folder_);
   cache_folder.create ();
-      
+
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Data folder
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -138,39 +134,10 @@ _init ()
 
   if (mobius_data_dir)
     data_folder_ = mobius_data_dir;
-
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // Database
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  const std::string db_path = join_path (config_folder_, id_ + ".sqlite");
-  connection_pool_.set_path (db_path);
-
-  auto db = connection_pool_.get_database ();
-  auto transaction = db.new_transaction ();
-
-  db.execute ("PRAGMA foreign_keys = ON;");
-
-  db.execute (
-      "CREATE TABLE IF NOT EXISTS config"
-                            "(var TEXT PRIMARY KEY,"
-                           "value TEXT NULL);"
-         );
-  
-  transaction.commit ();
 }
 
 } // namespace
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Get database object for current thread
-//! \return Database object
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-mobius::database::database
-get_database ()
-{
-  return connection_pool_.get_database ();
-}
-
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Constructor
 //! \see https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
@@ -189,26 +156,35 @@ application::start ()
   mobius::core::log log (__FILE__, __FUNCTION__);
   log.info (__LINE__, "starting application");
 
-  // check if application is already running
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Check if application is already running
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   if (is_running_)
     throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("application is already running"));
 
-  // search installed C++ extensions
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Load C++ extensions
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   log.info (__LINE__, "loading C++ extensions");
+
   auto extensions_path = get_config_path ("extensions");
   auto folder = mobius::io::new_folder_by_path (extensions_path);
-  
+
   for (auto& c : folder.get_children ())
     {
       if (c.is_file () && c.get_extension () == "so")
-        extensions_.load (c.get_path ());
+        extensions_.load (extensions_path + '/' + c.get_name ());
     }
 
-  // start extensions
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Start C++ extensions
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   for (auto& e : extensions_.get_extensions ())
     e.start ();
 
-  // set application running
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Set application running
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   log.info (__LINE__, "application started");
   is_running_ = true;
 }
@@ -221,11 +197,15 @@ application::stop ()
 {
   mobius::core::log log (__FILE__, __FUNCTION__);
 
-  // check if application is running
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Check if application is running
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   if (!is_running_)
     throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("application is not running"));
 
-  // stop extensions
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Stop C++ extensions
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   log.info (__LINE__, "unloading C++ extensions");
 
   for (auto& e : extensions_.get_extensions ())
@@ -234,13 +214,15 @@ application::stop ()
   // remove extensions
   //extensions_.clear ();
 
-  // set running false  
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Set running false
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   log.info (__LINE__, "application stopped");
   is_running_ = false;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get application ID
+//! \brief Get application ID
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_id () const
@@ -249,7 +231,7 @@ application::get_id () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get application name
+//! \brief Get application name
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_name () const
@@ -258,7 +240,7 @@ application::get_name () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get application version
+//! \brief Get application version
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_version () const
@@ -267,7 +249,7 @@ application::get_version () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get application title
+//! \brief Get application title
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_title () const
@@ -276,7 +258,7 @@ application::get_title () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get application copyright
+//! \brief Get application copyright
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_copyright () const
@@ -285,7 +267,7 @@ application::get_copyright () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get application host OS
+//! \brief Get application host OS
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_os_name () const
@@ -294,178 +276,36 @@ application::get_os_name () const
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get data path
+//! \brief Get data path
 //! \param path relative path
 //! \return absolute path to data_folder sub-path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_data_path (const std::string& path) const
 {
-  return join_path (data_folder_, path);
+  return _join_path (data_folder_, path);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get config path
+//! \brief Get config path
 //! \param path relative path
 //! \return absolute path to config_folder sub-path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_config_path (const std::string& path) const
 {
-  return join_path (config_folder_, path);
+  return _join_path (config_folder_, path);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get cache path
+//! \brief Get cache path
 //! \param path relative path
 //! \return absolute path to cache_folder sub-path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 application::get_cache_path (const std::string& path) const
 {
-  return join_path (cache_folder_, path);
+  return _join_path (cache_folder_, path);
 }
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create new connection to application database
-//! \return new connection object
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-mobius::database::connection
-application::new_connection ()
-{
-  return connection_pool_.acquire ();
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create new transaction to application database
-//! \return new database transaction
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-mobius::database::transaction
-application::new_transaction ()
-{
-  auto db = get_database ();
-  return db.new_transaction ();
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief check if config var exists
-//! \param var var name
-//! \return true/false
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-bool
-application::has_config (const std::string& var) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
-  statement.bind (1, var);
-
-  return statement.fetch_row ();
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get config var
-//! \param var var name
-//! \return int var
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-int
-application::get_config_int (const std::string& var) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
-  statement.bind (1, var);
-  
-  int value = 0;
-
-  if (statement.fetch_row ())
-    value = statement.get_column_int (0);
-  
-  return value;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get config var
-//! \param var var name
-//! \return double var
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-double
-application::get_config_double (const std::string& var) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
-  statement.bind (1, var);
-  
-  double value = 0.0;
-
-  if (statement.fetch_row ())
-    value = statement.get_column_double (0);
-  
-  return value;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get config var
-//! \param var var name
-//! \return string var
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-std::string
-application::get_config_string (const std::string& var) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
-  statement.bind (1, var);
-  
-  std::string value;
-
-  if (statement.fetch_row ())
-    value = statement.get_column_string (0);
-  
-  return value;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief set config int var
-//! \param var var name
-//! \param v int value
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
-application::set_config (const std::string& var, int value) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("INSERT OR REPLACE INTO config VALUES (?, ?)");
-  statement.bind (1, var);
-  statement.bind (2, value);
-  statement.execute ();
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief set config double var
-//! \param var var name
-//! \param v double value
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
-application::set_config (const std::string& var, double value) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("INSERT OR REPLACE INTO config VALUES (?, ?)");
-  statement.bind (1, var);
-  statement.bind (2, value);
-  statement.execute ();
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief set config string var
-//! \param var var name
-//! \param v string value
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
-application::set_config (const std::string& var, const std::string& value) const
-{
-  auto db = get_database ();
-  auto statement = db.new_statement ("INSERT OR REPLACE INTO config VALUES (?, ?)");
-  statement.bind (1, var);
-  statement.bind (2, value);
-  statement.execute ();
-}
-
-} // namespace core
-} // namespace mobius
+} // namespace mobius::core
