@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -17,13 +17,13 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include "uri.h"
 #include "path.h"
+#include <mobius/charset.h>
 #include <mobius/string_functions.h>
 #include <mobius/exception.inc>
+#include <cstdlib>
 #include <regex>
 #include <stdexcept>
 
-#include <iostream>
-
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Syntax:
 //
@@ -130,7 +130,8 @@ _decode_triplets (const std::string& s)
 
       if (pos + 2 < length)
         {
-          tmp += char (stoi (s.substr (pos + 1, 2), nullptr, 16));
+          std::string v = s.substr (pos + 1, 2);
+          tmp += char (strtol (v.c_str (), nullptr, 16));
           begin = pos + 3;
         }
       else
@@ -207,14 +208,14 @@ uri::uri (const std::string& value)
 
   // Set attributes
   scheme_ = mobius::string::tolower (_decode_triplets (match[2].str ()));
-  query_ = _decode_triplets (match[7].str ());
-  fragment_ = _decode_triplets (match[9].str ());
+  query_ = match[7].str ();
+  fragment_ = match[9].str ();
 
   // Parse authority
   _set_authority (_decode_triplets (match[4].str ()));
 
   // If it is not a relative URI, remove dot segments - RFC 3986 - section 6.2.2.3
-  std::string path = _decode_triplets (match[5].str ());
+  std::string path = match[5].str ();
 
   if ((!scheme_.empty () || !host_.empty ()) && !path.empty ())
     path = remove_dot_segments (path);
@@ -250,9 +251,9 @@ uri::uri (
    password_ (password),
    host_ (host),
    port_ (port),
-   path_ (path),
-   query_ (query),
-   fragment_ (fragment)
+   path_ (_encode_triplets (path, ALLOWED_PATH)),
+   query_ (_encode_triplets (query, ALLOWED_QUERY)),
+   fragment_ (_encode_triplets (fragment, ALLOWED_FRAGMENT))
 {
   _normalize ();
 }
@@ -308,18 +309,18 @@ uri::_normalize ()
       value_ += _encode_triplets (authority_, ALLOWED_AUTHORITY);
     }
 
-  value_ += _encode_triplets (path_, ALLOWED_PATH);
+  value_ += path_;
 
   if (!query_.empty ())
     {
       value_ += "?";
-      value_ += _encode_triplets (query_, ALLOWED_QUERY);
+      value_ += query_;
     }
 
   if (!fragment_.empty ())
     {
       value_ += "#";
-      value_ += _encode_triplets (fragment_, ALLOWED_FRAGMENT);
+      value_ += fragment_;
     }
 }
 
@@ -400,6 +401,48 @@ uri::_set_authority (const std::string& value)
   port_ = _decode_triplets (port);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get path
+//! \param encoding String encoding
+//! \return URI path
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+uri::get_path (const std::string& encoding) const
+{
+  if (encoding.empty ())
+    return path_;
+
+  return conv_charset_to_utf8 (_decode_triplets (path_), encoding);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get query string
+//! \param encoding String encoding
+//! \return URI query string
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+uri::get_query (const std::string& encoding) const
+{
+  if (encoding.empty ())
+    return query_;
+  
+  return conv_charset_to_utf8 (_decode_triplets (query_), encoding);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get fragment
+//! \param encoding String encoding
+//! \return URI fragment
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+uri::get_fragment (const std::string& encoding) const
+{
+  if (encoding.empty ())
+    return fragment_;
+
+  return conv_charset_to_utf8 (_decode_triplets (fragment_), encoding);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get parent URI
 //! \return Parent URI
@@ -407,21 +450,56 @@ uri::_set_authority (const std::string& value)
 uri
 uri::get_parent () const
 {
-  uri value;
-  std::string::size_type pos = path_.rfind ('/');
+  uri parent;
+
+  auto pos = value_.rfind ('/');
 
   if (pos != std::string::npos)
-    {
-      value = uri (scheme_,
-                   username_,
-                   password_,
-                   host_,
-                   port_,
-                   path_.substr (0, pos)
-                  );
-    }
+    parent = uri (value_.substr (0, pos));
 
-  return value;
+  return parent;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get sibling URI by name
+//! \param filename Sibling file name
+//! \return Sibling URI
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+uri
+uri::get_sibling_by_name (const std::string& filename) const
+{
+  path p (path_);
+  path cpath = p.get_sibling_by_name (filename);
+
+  return uri (scheme_, username_, password_, host_, port_, cpath.get_value ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get sibling URI by extension
+//! \param ext Sibling extension
+//! \return Sibling URI
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+uri
+uri::get_sibling_by_extension (const std::string& ext) const
+{
+  path p (path_);
+  path cpath = p.get_sibling_by_extension (ext);
+
+  return uri (scheme_, username_, password_, host_, port_, cpath.get_value ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get child URI by name
+//! \param name Child name
+//! \return Child URI
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+uri
+uri::get_child_by_name (const std::string& name) const
+{
+  path p (path_);
+  path cpath = p.get_child_by_name (name);
+
+  return uri (scheme_, username_, password_, host_, port_, cpath.get_value ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -442,10 +520,11 @@ uri::get_child_by_path (const std::string& subpath) const
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get filename from URI
+//! \param encoding String encoding
 //! \return Filename
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
-uri::get_filename () const
+uri::get_filename (const std::string& encoding) const
 {
   std::string::size_type pos = path_.rfind ('/');
   std::string value;
@@ -453,7 +532,10 @@ uri::get_filename () const
   if (pos != std::string::npos)
     value = path_.substr (pos + 1);
 
-  return value;
+  if (encoding.empty ())
+    return value;
+
+  return conv_charset_to_utf8 (_decode_triplets (value), encoding);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -552,7 +634,7 @@ join (const uri& base, const uri& rel)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Check whether two URI objects are equal
+//! \brief Test if two URI objects are equal
 //! \param lhs URI object
 //! \param rhs URI object
 //! \return true if lhs == rhs
@@ -563,6 +645,18 @@ operator== (const uri& lhs, const uri& rhs)
   return lhs.get_value () == rhs.get_value ();
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Test if two URI objects are different
+//! \param lhs URI object
+//! \param rhs URI object
+//! \return true if lhs != rhs
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+operator!= (const uri& lhs, const uri& rhs)
+{
+  return ! (lhs.get_value () == rhs.get_value ());
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check whether one URI object is less than another one
 //! \param lhs URI object
