@@ -26,8 +26,7 @@
 #include "module.h"
 #include "application.h"
 #include "item.h"
-#include "password.h"
-#include "password_hash.h"
+#include "evidence.h"
 #include "database/connection.h"
 #include "database/transaction.h"
 
@@ -356,7 +355,7 @@ tp_f_get_passwords (model_case_o *self, PyObject *)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_passwords (),
-               pymobius_model_password_to_pyobject
+               pymobius_model_evidence_to_pyobject
             );
     }
   catch (const std::exception& e)
@@ -382,7 +381,7 @@ tp_f_get_password_hashes (model_case_o *self, PyObject *)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_password_hashes (),
-               pymobius_model_password_hash_to_pyobject
+               pymobius_model_evidence_to_pyobject
             );
     }
   catch (const std::exception& e)
